/*
Part of the Bungie.net Myth2 Metaserver source code
Copyright (c) 1997-2002 Bungie Studios
Refer to the file "License.txt" for details
*/

 
 /*
  * The metaserver code changes that fall outside the original Bungie.net metaserver code 
  * license were written and are copyright 2002, 2003 of the following individuals:
  *
  * Copyright (c) 2002, 2003 Alan Wagner
  * Copyright (c) 2002 Vishvananda Ishaya
  * Copyright (c) 2003 Bill Keirstead
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  */


// this file has been gutted :)
#include "cseries.h"
#include <string.h>

#ifdef linux
#include <time.h>
#endif

#include "authentication.h"

#include "md5.h"

/* -------- code */

#ifdef SERVER
static authentication_token global_guest_token= {0};

long get_current_time(
	void)
{
	return time(NULL);
}

void test_authentication_tokens(
	void)
{
	return;
}

void get_guest_user_token(
	authentication_token token)
{
	memcpy(token, global_guest_token, sizeof(authentication_token));

	return;
}

boolean guest_token(
	authentication_token *token)
{
	return (memcmp(token, global_guest_token, sizeof(authentication_token))==0);
}

void generate_authentication_token(
	long client_address, 
	long user_id,
	long expiration_time, 
	authentication_token token)
{
	char *p = (char *) token;

	// Copy address/expiration_time/user_id into token
	memcpy(p, &client_address, sizeof(client_address));
	p += sizeof(client_address);
	memcpy(p, &expiration_time, sizeof(expiration_time));
	p += sizeof(expiration_time);
	memcpy(p, &user_id, sizeof(user_id));

	return;
}

boolean authenticate_token(
	long client_address, 
	long current_time, 
	long *user_id,
	authentication_token token)
{
	char *p = (char *) token;
	boolean valid = FALSE;
	long c_address, e_time, u_id;

	// start by reading in the client address from the token and compare it
	// with the one we were passed
	memcpy(&c_address, p, sizeof(c_address));
	p += sizeof(c_address);
	if(c_address != client_address) {
	    // Address doesn't match - return false
	    return valid;
	}
	// if address was valid, check expiration date
	memcpy(&e_time, p, sizeof(e_time));
	p += sizeof(e_time);
	if(current_time > e_time) {
	    // current time is passed the expiration time - token has 'expired'
	    return valid;
	}
	// if both address & expiration time were OK, then copy the user_id and return true
	memcpy(&u_id, p, sizeof(u_id));
	if(u_id <= 0) {
	    // user_id is negative - something's not right
	    return valid;
	}
	*user_id = u_id;
	valid = TRUE;

	return valid;

}

void get_random_salt(
	byte *salt)
{
	int index;
	static boolean first_time= TRUE;
	
	if(first_time)
	{
		srand(machine_tick_count());
		first_time= FALSE;
	}

	for(index= 0; index<MAXIMUM_SALT_SIZE; ++index)
	{
		salt[index]= (byte) random();
	}
	
	return;
}
#endif

void encrypt_password(
	char *password,
	unsigned char *salt, 
	char *result, // must be of EXTENDED_MAXIMUM_ENCRYPTED_PASSWORD_SIZE size
	short type)
{
	switch (type)
	{
		default:
		case _plaintext_encryption:
			strcpy(result, password);
			break;

		case _simple_encryption:
			// this is the original Bungie algorithm that is used by Myth clients that do not support MD5 encryption - Bungie did not release the code for it...
			#warning compiling without support for Myth password encryption
			// if you implement support for this encryption type, change the determine_authentication_type_for_login()
			// function in users_new/main.c, which is currently set to ignore _simple_encryption
			break;

#ifdef ENABLE_MD5_SUPPORT
		case _md5_encryption:
			{
				int index;
				char buffer[EXTENDED_MAXIMUM_ENCRYPTED_PASSWORD_SIZE] = {0};
				strcpy(buffer, password);
				// xor with the salt first, to prevent dictionary lookups
				for (index = 0; index < MAXIMUM_SALT_SIZE && buffer[index] != '\0'; index++)
				{
					buffer[index] = buffer[index] ^ salt[index];
				}
				strcpy(result, md5sum(buffer));
			}
			break;
#endif
	}

	return;
}

// returns TRUE on match, FALSE otherwise
boolean passwords_match(
	char *password,
	char *encrypted_password,
	unsigned char *salt,
	short type)
{
	boolean success = FALSE;
	char password_buffer[EXTENDED_MAXIMUM_ENCRYPTED_PASSWORD_SIZE] = {0};
	int size = MAXIMUM_ENCRYPTED_PASSWORD_SIZE;

#ifdef ENABLE_MD5_SUPPORT
	if (type == _md5_encryption)
		size = MD5_MAXIMUM_ENCRYPTED_PASSWORD_SIZE;
#endif
	if (type == _plaintext_encryption)
		size = strlen(password) + 1;

	encrypt_password(password, salt, password_buffer, type);
	if (!memcmp(password_buffer, encrypted_password, size))
		success = TRUE;

	return success;
}

