/*
Part of the Bungie.net Myth2 Metaserver source code
Copyright (c) 1997-2002 Bungie Studios
Refer to the file "License.txt" for details
*/

#ifndef __GAME_SEARCH_PACKETS_H
#define __GAME_SEARCH_PACKETS_H

#include <byte_swapping.h>
#include <cseries.h>
#include <metaserver_common_structs.h>

#ifdef BN2_DEMOVERSION
#define DEFAULT_GAME_SEARCH_PORT	7981
#else
#define DEFAULT_GAME_SEARCH_PORT	7980
#endif

enum
{
	_net_game_in_progress_bit = 0,
	_net_game_has_password_bit,
	_net_game_invalid_bit,
	_net_game_is_tournament_round_bit,
	_net_game_is_closed_bit,
	NUMBER_OF_NET_BITS,

	_net_game_in_progress_flag = FLAG(_net_game_in_progress_bit),
	_net_game_has_password_flag = FLAG(_net_game_has_password_bit),
	_net_game_invalid_flag = FLAG(_net_game_invalid_bit),
	_net_game_is_tournament_round_flag = FLAG(_net_game_is_tournament_round_bit),
	_net_game_is_closed_flag = FLAG(_net_game_is_closed_bit)
};

/* flags found in: game->description.parameters.option_flags */
/* Note: changing this stuff can prove deadly, as it seems we get these flags as-is in a packet from the client */
#define GAME_OPTION_RANDOM_ENDGAME_COUNTDOWN      (1 << 0)	// game ends with random, undisplayed countdown
#define GAME_OPTION_ALLOW_MULTIPLAYER_TEAMS       (1 << 1)	// allow teams of more than one player
#define GAME_OPTION_LIMITED_VISIBILITY            (1 << 2)	// can't see guys too far away
#define GAME_OPTION_NO_INGAME_RANKING             (1 << 3)	// don't display ingame (F7) rankings
#define GAME_OPTION_ALLOW_UNIT_TRADING            (1 << 4)
#define GAME_OPTION_ALLOW_VETERANS                (1 << 5)
#define GAME_OPTION_CONTINUATION                  (1 << 6)	// server does not use, it seems
#define GAME_OPTION_COOPERATIVE                   (1 << 7)	// server does not use, it seems
#define GAME_OPTION_RANDOM_TEAMS                  (1 << 8)	// server does not use, it seems
#define GAME_OPTION_LIMITED_TERRAIN_VISIBILITY    (1 << 9)	// server does not use, it seems
#define GAME_OPTION_CAMERA_TRACKING_INITIALLY_ON  (1 << 10)	// server does not use, it seems
#define GAME_OPTION_PAUSED                        (1 << 11)	// server does not use, it seems
#define GAME_OPTION_LOCAL_TIME_PAUSED             (1 << 12)	// server does not use, it seems
#define GAME_OPTION_ALLOW_ALLIANCES               (1 << 13)
#define GAME_OPTION_ALLOW_OVERHEAD_MAP            (1 << 14)
#define GAME_OPTION_ORDER_GAME                    (1 << 15)
#define GAME_OPTION_SERVER_IS_OBSERVER            (1 << 16)
#define GAME_OPTION_RESTORE_VETERANS              (1 << 17)
#define GAME_OPTION_PATCH_VERSION_BIT1            (1 << 18)
#define GAME_OPTION_PATCH_VERSION_BIT2            (1 << 19)
#define GAME_OPTION_DEATHMATCH                    (1 << 20)
#define GAME_OPTION_VTFL                          (1 << 21)
#define GAME_OPTION_ANTICLUMP                     (1 << 22)
#define GAME_OPTION_TEAM_CAPTAIN_CHOSEN           (1 << 23)
#define GAME_OPTION_METASERVER_HOSTED             (1 << 24)
#define GAME_OPTION_VERSION_13                    (1 << 25)	// version flag for replaying old films
#define GAME_OPTION_VERSION_14                    (1 << 26)	// version flag for replaying old films
#define GAME_OPTION_VERSION_14X                   (1 << 27)	// version flag for replaying old films
#define GAME_OPTION_VERSION_150                   (1 << 28)	// version flag for replaying old films
#define GAME_OPTION_HAS_CAPTAIN                   (1 << 29)	// in-game, for morale stuff

#define NUM_GAME_OPTIONS							30

enum
{ // Myth II game types...
	_body_count = 0,
	_steal_the_bacon,
	_last_man_on_the_hill,
	_scavenger_hunt,
	_flag_rally,
	_capture_the_flag,
	_balls_on_parade,
	_territories,
	_captures,
	_king_of_the_hill,
	_stampede,
	_assassination,
	_hunting,
	_custom_scoring, // map actions determine endgame
	_king_of_the_hill_tfl // [must have time limit]most time on hill wins (was royalty)
};

enum
{
	_gs_login_packet,
	_gs_update_packet,
	_gs_query_packet,
	_gs_query_response_packet,
	NUMBER_OF_GS_PACKETS
};

enum
{
	_add_new_game,
	_change_game_info,
	_remove_game
};

struct gs_login_packet
{
	long room_id;
};

struct gs_update_packet
{
	int type_of_update;
	long room_id;
	boolean game_is_ranked;
	struct metaserver_game_aux_data	aux_data;
	int game_data_length;
	struct metaserver_game_description game;
};

struct gs_query_packet
{
	unsigned long player_id;
	short game_type;
	short game_scoring;
	short unit_trading;
	short veterans;
	short teams;
	short alliances;
	short enemy_visibility;
	word pad;
};

struct gs_query_response_header
{
	// contains the number of responses
	// and the player to whom the responses are directed
	int number_of_responses;
	unsigned long player_id;
};

struct gs_query_response_segment
{
	int segment_length;
	long room_id;
	boolean	game_is_ranked;
	struct metaserver_game_aux_data	aux_data;
	int game_data_length;
	struct metaserver_game_description game;
};

struct g_bs_data
{
	byte_swap_code * bs_code;
	short length;
};

short build_gs_login_packet(char *buffer, long room_id);
short build_gs_update_packet(char *buffer, int type, long room_id, boolean game_is_ranked, 
	struct metaserver_game_aux_data *aux_data, int game_data_length, 
	struct metaserver_game_description *game);
short build_gs_query_packet(char *buffer, unsigned long player_id, char *game_name, char *map_name, 
	short game_scoring, short unit_trading, short veterans, short teams, 
	short alliances, short enemy_visibility);
int build_gs_query_response_packet(char *buffer, unsigned long user_id, struct query_response *qr);
short build_gs_update_delete_packet(char *buffer, int type, long game_id, long room_id, long creating_player_id);
boolean send_game_search_packet(int socket, char *buffer, short length);

boolean byte_swap_game_search_packet(char *buffer, boolean outgoing);
void byte_swap_game_data(char *buffer);

#endif
