/*
Part of the Bungie.net Myth2 Metaserver source code
Copyright (c) 1997-2002 Bungie Studios
Refer to the file "License.txt" for details
*/

 /*
  * The metaserver code changes that fall outside the original Bungie.net metaserver code 
  * license were written and are copyright 2002, 2003 of the following individuals:
  *
  * Copyright (c) 2002, 2003 Alan Wagner
  * Copyright (c) 2002 Vishvananda Ishaya
  * Copyright (c) 2003 Bill Keirstead
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  */

#include "cseries.h"
#include <string.h>

#include "environment.h"
#include "bungie_net_player.h"
#include "room_list_file.h"

// Myrd - this is better
struct {
	const char *name;
	short supported_application_flags;
} room_types[] = {
	{ "MYTH"     , _myth1_game_type_flag | _myth2_game_type_flag | _myth3_game_type_flag },
	{ "MYTH1"    , _myth1_game_type_flag },
	{ "MYTH2"    , _myth2_game_type_flag },
	{ "MYTH3"    , _myth3_game_type_flag },
	{ "MARATHON" , _marathon_game_type_flag },
	{ "JCHAT"    , _jchat_game_type_flag },
};
#define NUMBER_OF_ROOM_TYPES (sizeof(room_types)/sizeof(room_types[0]))
// Myrd - end

/* ------ local prototypes */
static struct room_data *add_room(struct room_data *rooms, short supported_application_flags,
	short room_id, short ranked_room, short country_code, short minimum_caste,
	short maximum_caste, short tournament_room);

/* ------ code */
struct room_data *load_room_list(void)
{
	char filename[256] = {0};
	FILE *fp = NULL;
	struct room_data * rooms= NULL;

	strcpy(filename, get_rooms_list_file());
	fp= fopen(filename, "r");
	if(fp)
	{
		char name_list[256];
		int room_identifier;
		int ranked;
		int country_code;
		int minimum_caste;
		int maximum_caste;
		int tournament_room;
		
		while (fscanf(fp, "%s %d %d %d %d %d %d \n", name_list, &room_identifier, &ranked,
			&country_code, &minimum_caste, &maximum_caste, &tournament_room) != EOF)
		{
			short supported_application_flags= get_supported_application_flags_from_name_list(name_list);

			if (supported_application_flags != 0)
			{
				rooms= add_room(rooms, supported_application_flags, room_identifier, ranked, 
					country_code, minimum_caste, maximum_caste, tournament_room);
			} else {
				printf("Unrecognized name list in room list file '%s'\n", name_list);
			}
		}
		fclose(fp);
	}
	else
	{
		printf("No rooms list file found! The userd will not be able to load any rooms!\n");
		printf("bailing out...\n");
		exit(1);
	}
	return rooms;
}	

// Myrd - new functions
short get_supported_application_flags_from_name_list(
	char *name_list)
{
	int i;
	char *temp, *token, *string = name_list;
	short supported_application_flags = 0;

	while (1)
	{
		token = strtok_r(string, "|" , &temp);
		if (!token)
			break;
		for (i = 0; i < NUMBER_OF_ROOM_TYPES; i++)
		{
			if (!strcasecmp(token, room_types[i].name))
			{
				supported_application_flags |= room_types[i].supported_application_flags;
				break;
			}
		}
		string = temp;
	}

	return supported_application_flags;
}

char *get_name_list_from_supported_application_flags(
	short supported_application_flags)
{
	int i;
	static char name_list[256];

	name_list[0] = '\0';
	for (i = 0; i < NUMBER_OF_ROOM_TYPES; i++)
	{
		if (room_types[i].supported_application_flags == (supported_application_flags & room_types[i].supported_application_flags))
		{
			supported_application_flags &= ~(room_types[i].supported_application_flags);
			if (name_list[0] != '\0')
				strcat(name_list, "|");
			strcat(name_list, room_types[i].name);
		}
	}

	return name_list;
}

short get_application_type_from_name(
	char *name)
{
	short i, application_type = 0; // 0 is invalid

	for (i = 0; i < NUMBER_OF_ROOM_TYPES; i++)
	{
		if (!strcasecmp(name, room_types[i].name)) 
		{
			// special case: old Myth2 1.3.x clients reporting themselves as "MYTH"
			application_type = (i == 0 ? _myth2_game_type_flag : room_types[i].supported_application_flags);
			break;
		}
	}

	return application_type;
}
// Myrd - end

boolean save_room_list(
	struct room_data *rooms,
	char *filename)
{
	boolean success= FALSE;
	FILE *fp;

	fp= fopen(filename, "w+");
	if(fp) 
	{	
		struct room_data *room= rooms;	

		while(room) 
		{
			fprintf(fp, "%s %d %d %d %d %d %d\n",
			get_name_list_from_supported_application_flags(room->supported_application_flags),
			room->room_identifier,
			room->ranked_room,
			room->country_code,
			room->minimum_caste,
			room->maximum_caste,
			room->tournament_room);
			room= room->next;
		}
		fclose(fp);
		success= TRUE;
	}

	return success;
}

void list_room_templates(
	struct room_data *room)
{
	printf("Game\tRoomID\tRanked\tCountry\tMin Caste\tMax Caste\tTournament Room#\n");
	while(room) 
	{
		printf("%s %d %d %d %d %d %d\n",
			get_name_list_from_supported_application_flags(room->supported_application_flags),
			room->room_identifier,
			room->ranked_room,
			room->country_code,
			room->minimum_caste,
			room->maximum_caste,
			room->tournament_room);
		room= room->next;
	}

	return;
}

struct room_data *delete_room_template(
	struct room_data *rooms,
	short supported_application_flags,
	short room_identifier)
{
	if(supported_application_flags != 0)
	{
		struct room_data *room, *previous;
		
		previous= NULL;
		room= rooms;
		while(room && room->supported_application_flags!=supported_application_flags && room->room_identifier != room_identifier)
		{
			previous= room;
			room= room->next;
		}
		
		if(room)
		{
			if(previous)
			{
				previous->next= room->next;
			} else {
				rooms= room->next;
			}
			free(room);
			printf("Room deleted!\n");
		} else {
			printf("Room not found!\n");
		}
	} else {
		printf("No supported client flags!\n");
	}

	return rooms;
}

struct room_data *add_or_update_room(
	struct room_data *rooms,
	short supported_application_flags,
	short room_identifier,
	short ranked_room,
	short country_code,
	short minimum_caste,
	short maximum_caste,
	short tournament_room)
{
	if(supported_application_flags != 0) 
	{
		struct room_data *room;

		room= rooms;
		while(room) 
		{
			if(room->supported_application_flags==supported_application_flags && 
				room->room_identifier==room_identifier) 
			{
				room->ranked_room= ranked_room;
				room->country_code= country_code;
				room->minimum_caste= minimum_caste;
				room->maximum_caste= maximum_caste;
				room->tournament_room = tournament_room;
				printf("Room updated!\n");
				break;
			}

			room= room->next;
		}

		if(!room) 
		{
			rooms= add_room(rooms, supported_application_flags, room_identifier, ranked_room,
				country_code, minimum_caste, maximum_caste, tournament_room);
		}
	} else {
		printf("No supported client flags!\n");
	}

	return rooms;
}

/* --------- local code */
static struct room_data *add_room(
	struct room_data *rooms,
	short supported_application_flags,
	short room_id,
	short ranked_room,
	short country_code,
	short minimum_caste,
	short maximum_caste,
	short tournament_room)
{
	struct room_data *room= (struct room_data *) malloc(sizeof(struct room_data));

	if(room) 
	{
		memset(room, 0, sizeof(struct room_data));
		room->supported_application_flags= supported_application_flags;
		room->room_identifier= room_id;
		room->ranked_room= ranked_room;
		room->country_code= country_code;
		room->minimum_caste= minimum_caste;
		room->maximum_caste= maximum_caste;
		room->tournament_room = tournament_room;
		room->used= FALSE;

		if(!rooms) 
		{
			rooms= room;
		} else {
			struct room_data *previous= rooms;

			while(previous->next != NULL) previous= previous->next;
			previous->next= room;
		}
	} else {
		printf("Out of memory adding room!\n");
	}

	return rooms;
}
