/*
Part of the Bungie.net Myth2 Metaserver source code
Copyright (c) 1997-2002 Bungie Studios
Refer to the file "License.txt" for details
*/

#ifndef __ROOM_GLOBALS__
#define __ROOM_GLOBALS__

#include <netinet/in.h>
#include "room_packets.h"

// handy macros

#define CREATE(result, type, number)  do { if ((number) * sizeof(type) <= 0) printf("Error: Zero bytes or less requested at %s:%d.", __FILE__, __LINE__); if (!((result) = (type *) calloc ((number), sizeof(type)))) { perror("Error: malloc failure"); exit(1); } } while(0)

// Myrd - one element from the list of app types that we got from server
struct application_type_data {
	long time_stamp;
	long host;
	short application_type;
	char username[MAXIMUM_METASERVER_USERNAME+1];
	struct application_type_data *next;
};
// Myrd - end change

struct global_room_data
{
	boolean logged_into_userd;
	boolean	logged_into_game_search_engine;
	boolean logged_into_player_search_engine;

	time_t last_userd_ping;

	unsigned long flags;
	
	int userd_socket; // may be NONE if not logged in...
	int game_search_socket; // may be NONE if not logged in...
	int player_search_socket;
	short room_port;
	short room_identifier;
	short supported_application_flags;
	short seconds_before_update;
	short maximum_players_per_room;
	short unused[2];
	boolean ranked_room;
	boolean tournament_room;
	struct caste_breakpoint_data caste_breakpoints;
	struct overall_ranking_data overall_rank_data;
	char url_for_version_update[ROOM_MAXIMUM_UPDATE_URL_SIZE];
	char motd[ROOM_MAXIMUM_MOTD_SIZE];
	
	time_t launch_time;
	boolean squelched;
	
	long ticks_at_last_connection_attempt;
	long ticks_at_last_room_update;
	boolean game_search_initial_connect;
	boolean ready_for_search;
	
	fd_set readfds;

	struct application_type_data *pending_app_types; // Myrd

	struct player_room_list_data available_rooms[MAXIMUM_ROOMS];
	short available_room_count;

#ifdef MULTI_ROOM
	short country_code;
	short min_caste;
	short max_caste;
	unsigned long global_game_id;
	struct game_data *the_first_game;
	struct client_data *the_first_client;
	int server_socket;
	struct sockaddr_in server_address;
	int set_size;
	struct circular_queue userd_incoming;
	struct circular_queue game_search_incoming;
#endif
};

#ifdef MULTI_ROOM

#define room_globals			rooms[current_room]
#define first_client			rooms[current_room].the_first_client
#define first_game				rooms[current_room].the_first_game
#define room_global_game_id		rooms[current_room].global_game_id

extern struct global_room_data *rooms;
extern int current_room;
extern int number_of_rooms;
#else
extern struct global_room_data room_globals;
extern struct client_data *first_client;
#endif

void change_room_motd(char *motd);

#endif // __ROOM_GLOBALS__
