/*
Part of the Bungie.net Myth2 Metaserver source code
Copyright (c) 1997-2002 Bungie Studios
Refer to the file "License.txt" for details
*/

 /*
  * The metaserver code changes that fall outside the original Bungie.net metaserver code 
  * license were written and are copyright 2002, 2003 of the following individuals:
  *
  * Copyright (c) 2002, 2003 Alan Wagner
  * Copyright (c) 2002 Vishvananda Ishaya
  * Copyright (c) 2003 Bill Keirstead
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  *
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
  */

// this file has been gutted :)


// ALAN Begin: added headers to resolve improper parameter list declarations
#include "bungie_net_player.h"
#include "bungie_net_order.h"
// ALAN End
#include "games.h"

struct bungie_net_player_datum * find_player_struct_by_pid(long player_id, short player_count, struct bungie_net_player_datum **players);
struct bungie_net_game_standings * find_good_standings_for_game(short player_count, struct bungie_net_game_standings **all_standings);
boolean find_same_standings(struct bungie_net_game_standings *st1, struct bungie_net_game_standings* st2);


void bungie_net_game_evaluate(
	long host_player_id,
	short game_classification,
	short player_count,
	struct bungie_net_player_datum **bungie_net_players,
	struct bungie_net_order_datum **bungie_net_orders,
	struct bungie_net_game_standings **reported_standings)
{
    struct bungie_net_game_standings *current_standings;
    struct bungie_net_player_datum *current_player;
    /* get useable standings for game */
    current_standings = find_good_standings_for_game(player_count, reported_standings);
    if(current_standings == NULL)
        return;
    if(current_standings->players == NULL || current_standings->teams == NULL)
        return;
    /* temp hack - treat unranked games as if they were ranked */
    if(game_classification == _ranked_normal || game_classification == _unranked_normal)
    {
        /* go through reported_standings - only adjust winner and loser points */
	int i, current_team;
	long current_player_id;
	short place;
	for(i = 0; i < player_count; i++)
	{
	    current_player_id = current_standings->players[i].bungie_net_player_id;
	    current_team = current_standings->players[i].team_index;
	    current_player = find_player_struct_by_pid(current_player_id, player_count, bungie_net_players);
	    place = current_standings->teams[current_team].place;
            /* adjust games played, damage, and wins/losses/ties as appropriate */
            if(current_player != NULL)
	    {
	        /* adjust ranked score (for all game types) */
	        current_player->ranked_score.damage_inflicted += current_standings->players[i].points_killed;
	        current_player->ranked_score.damage_received += current_standings->players[i].points_lost;
		current_player->ranked_score.games_played++;
		/* adjust ranked score for specific game type */
	        current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].damage_inflicted += current_standings->players[i].points_killed;
	        current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].damage_received += current_standings->players[i].points_lost;
		current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].games_played++;
		if(place == 0)
		{
		    /* adjust wins & points for all game types */
		    (current_player->ranked_score.wins)++;
		    (current_player->ranked_score.points) += 3;
		    if(current_player->ranked_score.points > current_player->ranked_score.highest_points)
		        current_player->ranked_score.highest_points = current_player->ranked_score.points;
		    /* adjust wins & points for specific game type */
		    (current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].wins)++;
		    (current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].points) += 3;
		    if(current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].points > current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].highest_points)
		        current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].highest_points = current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].points;

		}
		else if(place == (current_standings->number_of_teams - 1))
		{
		    /* adjust losses and points for all game types */
		    (current_player->ranked_score.losses)++;
		    (current_player->ranked_score.points)--;
		    /* adjust losses for specific game type */
		    (current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].losses)++;
		    (current_player->ranked_scores_by_game_type[(current_standings->game_scoring)].points)--;
		}
	    }

	}

    }

    return;
}

struct bungie_net_player_datum * find_player_struct_by_pid(
    long player_id,
    short player_count,
    struct bungie_net_player_datum **players)
{
    int i;
    struct bungie_net_player_datum *current_player;
    if(players == NULL)
    {
	return NULL;
    }
    for(i = 0; i < player_count; i++)
    {
        // printf("i = %d\n", i);
        // current_player = *(players + i * sizeof(struct bungie_net_player_datum *));
	current_player = players[i];
	if(current_player->player_id == player_id)
	{
	    return current_player;
	}
    }
    return NULL;
}

/* function used to find a reliable set of standings for scoring a game */
struct bungie_net_game_standings * find_good_standings_for_game(
    short player_count,
    struct bungie_net_game_standings **all_standings)
{
    int i;
    struct bungie_net_game_standings *temp_standings, *good_standings = NULL;

    if(all_standings == NULL)
	return NULL;
    if(player_count == 1)
        return *all_standings;

    for(i = 0; i < player_count; i++)
    {
        temp_standings = all_standings[i];
	if(temp_standings == NULL)
	    continue;
	if(good_standings != NULL)
	{
	    if(find_same_standings(temp_standings, good_standings))
	        return good_standings;
	}
	else
	{
	    good_standings = temp_standings;
	}
    }
    /* if this is reached then there were no two standings the same */
    return NULL;

}

boolean find_same_standings(
    struct bungie_net_game_standings *st1,
    struct bungie_net_game_standings *st2)
{
    boolean rval = FALSE;
    if(st1 != NULL && st2 != NULL)
    {
        // yes, they are simmple checks
	// but good enough for gov't work =p
        if(st1->game_ended_code != st2->game_ended_code)
	    return rval;
        if(st1->version_number != st2->version_number)
	    return rval;
	if(st1->number_of_players != st2->number_of_players)
	    return rval;
	rval = TRUE;
    }
    return rval;
}

void scoring_datum_adjust_total(
	struct bungie_net_player_score_datum *score_by_game_types,
	struct bungie_net_player_score_datum *total_score)
{
	return;
}
